@@ -8,6 +8,7 @@ from typing import Callable, List, Optional, Tuple, Type, Union
 import numpy as np
 import tensorflow as tf
 import tensorflow_probability as tfp
+import tf_quant_finance.math.integration as tf_integration
 
 import zfit
 import zfit.z.numpy as znp
@@ -26,30 +27,61 @@ from .space import Space, convert_to_space, supports
 
 @supports()
 def auto_integrate(func, limits, n_axes=None, x=None, method="AUTO", dtype=ztypes.float,
-                   mc_sampler=tfp.mcmc.sample_halton_sequence,
-                   mc_options=None):
+                   mc_sampler=tfp.mcmc.sample_halton_sequence, max_draws=None, tol=None,
+                   mc_options=None, simpsons_options=None):
+    limits = convert_to_space(limits)
+
+    if n_axes is None:
+        n_axes = limits.n_obs
     if method == "AUTO":  # TODO unfinished, other methods?
-        method = "mc"
+        if n_axes == 1 and x is None:
+            method = "simpson"
+        else:
+            method = "mc"
     # TODO method
     if method.lower() == "mc":
         mc_options = mc_options or {}
         draws_per_dim = mc_options['draws_per_dim']
+        max_draws = mc_options.get('max_draws')
         integral = mc_integrate(x=x, func=func, limits=limits, n_axes=n_axes, method=method, dtype=dtype,
-                                mc_sampler=mc_sampler, draws_per_dim=draws_per_dim,
+                                mc_sampler=mc_sampler, draws_per_dim=draws_per_dim, max_draws=max_draws, tol=tol,
                                 importance_sampling=None)
+    elif method.lower() == 'simpson':
+        num_points = simpsons_options['draws_simpson']
+        integral = simpson_integrate(func=func, limits=limits, num_points=num_points)
+    else:
+        raise ValueError(f"Method {method} not a legal choice for integration method.")
     return integral
 
 
 # TODO implement numerical integration method
 def numeric_integrate():
     """Integrate `func` using numerical methods."""
-    integral = None
-    return integral
+    return None
+
+
+def simpson_integrate(func, limits, num_points):  # currently not vectorized
+    integrals = []
+    num_points += num_points % 2 + 1  # sanitize number of points
+    for space in limits:
+        lower, upper = space.rect_limits
+        if lower.shape[0] > 1:
+            raise ValueError("Vectorized spaces in integration currently not supported.")
+        lower = znp.array(lower)[0, 0]
+        upper = znp.array(upper)[0, 0]
+        tf.debugging.assert_all_finite((lower, upper),
+                                       "MC integration does (currently) not support unbound limits (np.infty) as given here:"
+                                       "\nlower: {}, upper: {}".format(lower, upper)
+                                       )
+        integrals.append(
+            tf_integration.simpson(func=func, lower=lower, upper=upper, num_points=num_points, dtype=znp.float64))
+    return znp.sum(integrals, axis=0)
 
 
 # @z.function
 def mc_integrate(func: Callable, limits: ztyping.LimitsType, axes: Optional[ztyping.AxesTypeInput] = None,
-                 x: Optional[ztyping.XType] = None, n_axes: Optional[int] = None, draws_per_dim: int = 20000,
+                 x: Optional[ztyping.XType] = None, n_axes: Optional[int] = None, draws_per_dim: int = 40000,
+                 max_draws=800_000, tol: float = 1e-6,
                  method: str = None,
                  dtype: Type = ztypes.float,
                  mc_sampler: Callable = tfp.mcmc.sample_halton_sequence,
@@ -72,9 +104,9 @@ def mc_integrate(func: Callable, limits: ztyping.LimitsType, axes: Optional[ztyp
     Returns:
         The integral
     """
+    tol = znp.array(tol, dtype=znp.float64)
     if axes is not None and n_axes is not None:
         raise ValueError("Either specify axes or n_axes")
-    limits = convert_to_space(limits)
 
     axes = limits.axes
     partial = (axes is not None) and (x is not None)  # axes, value can be tensors
@@ -92,7 +124,7 @@ def mc_integrate(func: Callable, limits: ztyping.LimitsType, axes: Optional[ztyp
                                        "\nlower: {}, upper: {}".format(lower, upper)
                                        )
 
-        n_samples = draws_per_dim
+        n_samples = draws_per_dim * n_axes
 
         chunked_normalization = zfit.run.chunksize < n_samples
         # chunked_normalization = True
@@ -107,41 +139,112 @@ def mc_integrate(func: Callable, limits: ztyping.LimitsType, axes: Optional[ztyp
 
         else:
             # TODO: deal with n_obs properly?
+            @z.function(wraps='tensor')
+            def cond(avg, error, std, ntot, i):
+                # return i < 3
+                return znp.logical_and(error > tol, ntot < max_draws)
 
-            samples_normed = mc_sampler(dim=n_axes, num_results=n_samples / 2,  # to decrease integration size
-                                        dtype=dtype)
-            samples = samples_normed * (upper - lower) + lower  # samples is [0, 1], stretch it
+            def body_integrate(avg, error, std, ntot, i):
+                ntot_old = ntot
+                ntot += n_samples
+                if partial:
+                    samples_normed = tfp.mcmc.sample_halton_sequence(dim=n_axes,
+                                                                     # sequence_indices=tf.range(ntot_old, ntot),
+                                                                     num_results=n_samples / 10,
+                                                                     # reduce, it explodes otherwise easily
+                                                                     # as we don't do adaptive now
+                                                                     # to decrease integration size
+                                                                     dtype=dtype,
+                                                                     randomized=False)
+                else:
+                    samples_normed = tfp.mcmc.sample_halton_sequence(dim=n_axes,
+                                                                     sequence_indices=tf.range(ntot_old, ntot),
+                                                                     # num_results=n_samples,  # to decrease integration size
+                                                                     dtype=dtype,
+                                                                     randomized=False)
+                samples = samples_normed * (upper - lower) + lower  # samples is [0, 1], stretch it
+                if partial:  # TODO(Mayou36): shape of partial integral?
+                    data_obs = x.obs
+                    new_obs = []
+                    xval = x.value()
+                    value_list = []
+                    index_samples = 0
+                    index_values = 0
+                    if len(xval.shape) == 1:
+                        xval = znp.expand_dims(xval, axis=1)
+                    for i in range(n_axes + xval.shape[-1]):
+                        if i in axes:
+                            new_obs.append(space.obs[index_samples])
+                            value_list.append(samples[:, index_samples])
+                            index_samples += 1
+                        else:
+                            new_obs.append(data_obs[index_values])
+                            value_list.append(znp.expand_dims(xval[:, index_values], axis=1))
+                            index_values += 1
+                    value_list = [tf.cast(val, dtype=dtype) for val in value_list]
+                    xval = PartialIntegralSampleData(sample=value_list,
+                                                     space=Space(obs=new_obs))
+                else:
+                    xval = samples
+                # convert rnd samples with value to feedable vector
+                reduce_axis = 1 if partial else None
+                y = func(xval)
+                # avg = znp.mean(y)
+                ifloat = tf.cast(i, dtype=tf.float64)
+                if partial:
+                    avg = znp.mean(y, axis=reduce_axis)
+                else:
+                    avg = avg / (ifloat + 1.) * ifloat + znp.mean(y, axis=reduce_axis) / (ifloat + 1.)
+                std = std / (ifloat + 1.) * ifloat + znp.std(y) / (ifloat + 1.)
+                ntot_float = znp.asarray(ntot, dtype=znp.float64)
 
-            if partial:  # TODO(Mayou36): shape of partial integral?
-                data_obs = x.obs
-                new_obs = []
-                x = x.value()
-                value_list = []
-                index_samples = 0
-                index_values = 0
-                if len(x.shape) == 1:
-                    x = znp.expand_dims(x, axis=1)
-                for i in range(n_axes + x.shape[-1]):
-                    if i in axes:
-                        new_obs.append(space.obs[index_samples])
-                        value_list.append(samples[:, index_samples])
-                        index_samples += 1
-                    else:
-                        new_obs.append(data_obs[index_values])
-                        value_list.append(znp.expand_dims(x[:, index_values], axis=1))
-                        index_values += 1
-                value_list = [tf.cast(val, dtype=dtype) for val in value_list]
-                x = PartialIntegralSampleData(sample=value_list,
-                                              space=Space(obs=new_obs))
+                # estimating the error of QMC is non-trivial
+                # (https://www.degruyter.com/document/doi/10.1515/mcma-2020-2067/html or
+                # https://stats.stackexchange.com/questions/533725/how-to-calculate-quasi-monte-carlo-integration-error-when-sampling-with-sobols)
+                # However, we use here just something that is in the right direction for the moment being. Therefore,
+                # we use the MC error as an upper bound as QMC is better/equal to MC (for our cases).
+                error_sobol = std * znp.log(ntot_float) ** n_axes / ntot_float
+                error_random = std / znp.sqrt(ntot_float)
+                error = znp.minimum(error_sobol, error_random) * 0.1  # heuristic factor from using QMC
+                return avg, error, std, ntot, i + 1
+
+            avg, error, std, ntot, i = [znp.array(0., dtype=znp.float64),
+                                        znp.array(9999., dtype=znp.float64),  # init value large, irrelevant
+                                        znp.array(0., dtype=znp.float64),
+                                        0,
+                                        0]
+            if partial:
+                avg, error, std, ntot, i = body_integrate(avg, error, std, ntot, i)
             else:
-                x = samples
+                avg, error, std, ntot, i = tf.while_loop(cond=cond, body=body_integrate,
+                                                         loop_vars=[avg, error, std, ntot, i])
+                from zfit import settings
+                if settings.get_verbosity() > 9:
+                    tf.print("i:", i, "   ntot:", ntot)
 
-            # convert rnd samples with value to feedable vector
-            reduce_axis = 1 if partial else None
-            avg = znp.mean(func(x), axis=reduce_axis)
             # avg = tfp.monte_carlo.expectation(f=func, samples=x, axis=reduce_axis)
             # TODO: importance sampling?
             # avg = tfb.monte_carlo.expectation_importance_sampler(f=func, samples=value,axis=reduce_axis)
+
+            def print_none_return():
+                from zfit import settings
+                if settings.get_verbosity() >= 0:
+                    tf.print(
+                        "Estimated integral error (", error,
+                        ") larger than tolerance (", tol,
+                        "), which is maybe not enough (but maybe it's also fine)."
+                        " You can (best solution) implement an anatytical integral (see examples in repo) or"
+                        " manually set a higher number on the PDF with 'update_integration_options'"
+                        " and increase the 'max_draws' (or adjust 'tol'). "
+                        "If partial integration is chosen, this can lead to large memory consumption."
+                        "This is a new warning checking the integral accuracy. It may warns too often as it is"
+                        " Work In Progress. If you have any observation on it, please tell us about it:"
+                        " https://github.com/zfit/zfit/issues/new/choose"
+                        "To suppress this warning, use zfit.settings.set_verbosity(-1)."
+                    )
+                return
+
+            tf.cond(error > tol, print_none_return, lambda: None)
         integral = avg * tf.cast(z.convert_to_tensor(space.rect_area()), dtype=avg.dtype)
         integrals.append(integral)
     return z.reduce_sum(integrals, axis=0)
@@ -214,7 +317,6 @@ def normalization_chunked(func, n_axes, batch_size, num_batches, dtype, space, x
                                              dtype=dtype,
                                              space=space, x=x, shape_after=shape_after)
 
-
             return dy, tape.gradient(value, variables)
 
         return value, grad_fn
@@ -391,10 +493,11 @@ class AnalyticIntegral:
             implemented_axes = set(self._integrals.keys())
         implemented_axes = sorted(implemented_axes, key=len, reverse=True)  # iter through biggest first
         for axes in implemented_axes:
-            limits_matched = []
-            for lim, integ in self._integrals[axes].items():
-                if integ.limits >= limits:
-                    limits_matched.append(lim)
+            limits_matched = [
+                lim
+                for lim, integ in self._integrals[axes].items()
+                if integ.limits >= limits
+            ]
 
             if limits_matched:  # one or more integrals available
                 return tuple(sorted(axes)), limits_matched
@@ -416,8 +519,7 @@ class AnalyticIntegral:
         axes, limits = self._get_max_axes_limits(limits=limits, out_of_axes=axes)
         axes = frozenset(axes)
         integrals = [self._integrals[axes][lim] for lim in limits]
-        integral_fn = max(integrals, key=lambda l: l.priority, default=None)
-        return integral_fn
+        return max(integrals, key=lambda l: l.priority, default=None)
 
     def register(self, func: Callable, limits: ztyping.LimitsType,
                  priority: int = 50, *,
@@ -507,6 +609,9 @@ class Integral:  # TODO analytic integral
 # to be "the" future integral class
 class Integration:
 
-    def __init__(self, mc_sampler, draws_per_dim):
+    def __init__(self, mc_sampler, draws_per_dim, tol, max_draws, draws_simpson):
+        self.tol = tol
+        self.max_draws = max_draws
         self.mc_sampler = mc_sampler
         self.draws_per_dim = draws_per_dim
+        self.draws_simpson = draws_simpson
