@@ -5,15 +5,15 @@ import tensorflow as tf
 import tensorflow_addons as tfa
 import tensorflow_probability as tfp
 
+import zfit.z.numpy as znp
+
 from .. import exception, z
 from ..core.data import Data, sum_samples
 from ..core.interfaces import ZfitPDF
 from ..core.sample import accept_reject_sample
 from ..core.space import supports
 from ..util import ztyping
-from ..util.exception import (ShapeIncompatibleError,
-                              SpecificFunctionNotImplemented,
-                              WorkInProgressError)
+from ..util.exception import ShapeIncompatibleError, WorkInProgressError
 from .functor import BaseFunctor
 
 
@@ -190,9 +190,8 @@ class FFTConvPDFV1(BaseFunctor):
         # the function as well
         area_ratios = (upper_sample - lower_sample) / (
                 limits_kernel.rect_upper - limits_kernel.rect_lower)
-        nbins_func_exact_max = tf.reduce_max(area_ratios * n)
-        nbins_func = tf.math.ceil(
-            nbins_func_exact_max)  # plus one and floor is like ceiling (we want more bins) with the
+        nbins_func_exact_max = znp.max(area_ratios * n)
+        nbins_func = znp.ceil(nbins_func_exact_max)  # plus one and floor is like ceiling (we want more bins) with the
         # guarantee that we add one bin (e.g. if we hit exactly the boundaries, we add one.
         nbins_kernel = n
         # n = max(n, npoints_scaling)
@@ -230,13 +229,13 @@ class FFTConvPDFV1(BaseFunctor):
         x_kernels = tf.linspace(lower_kernel, upper_kernel, tf.cast(nbins_kernel, tf.int32))
 
         x_func = tf.meshgrid(*tf.unstack(x_funcs, axis=-1), indexing='ij')
-        x_func = tf.transpose(x_func)
-        x_func_flatish = tf.reshape(x_func, (-1, self.n_obs))
+        x_func = znp.transpose(x_func)
+        x_func_flatish = znp.reshape(x_func, (-1, self.n_obs))
         data_func = Data.from_tensor(tensor=x_func_flatish, obs=self.obs)
 
         x_kernel = tf.meshgrid(*tf.unstack(x_kernels, axis=-1), indexing='ij')
-        x_kernel = tf.transpose(x_kernel)
-        x_kernel_flatish = tf.reshape(x_kernel, (-1, self.n_obs))
+        x_kernel = znp.transpose(x_kernel)
+        x_kernel_flatish = znp.reshape(x_kernel, (-1, self.n_obs))
         data_kernel = Data.from_tensor(tensor=x_kernel_flatish, obs=self.obs)
 
         y_func = self.pdfs[0].pdf(data_func, norm_range=False)
@@ -245,8 +244,8 @@ class FFTConvPDFV1(BaseFunctor):
         func_dims = [nbins_func] * self.n_obs
         kernel_dims = [nbins_kernel] * self.n_obs
 
-        y_func = tf.reshape(y_func, func_dims)
-        y_kernel = tf.reshape(y_kernel, kernel_dims)
+        y_func = znp.reshape(y_func, func_dims)
+        y_kernel = znp.reshape(y_kernel, kernel_dims)
 
         # flip the kernel to use the cross-correlation called `convolution function from TF
         # convolution = cross-correlation with flipped kernel
@@ -260,16 +259,16 @@ class FFTConvPDFV1(BaseFunctor):
         upper_func += kernel_shift
 
         # make rectangular grid
-        y_func_rect = tf.reshape(y_func, func_dims)
-        y_kernel_rect = tf.reshape(y_kernel, kernel_dims)
+        y_func_rect = znp.reshape(y_func, func_dims)
+        y_kernel_rect = znp.reshape(y_kernel, kernel_dims)
 
         # needed for multi dims?
         # if self.n_obs == 2:
         #     y_kernel_rect = tf.linalg.adjoint(y_kernel_rect)
 
         # get correct shape for tf.nn.convolution
-        y_func_rect_conv = tf.reshape(y_func_rect, (1, *func_dims, 1))
-        y_kernel_rect_conv = tf.reshape(y_kernel_rect, (*kernel_dims, 1, 1))
+        y_func_rect_conv = znp.reshape(y_func_rect, (1, *func_dims, 1))
+        y_kernel_rect_conv = znp.reshape(y_kernel_rect, (*kernel_dims, 1, 1))
 
         conv = tf.nn.convolution(
             input=y_func_rect_conv,
@@ -286,10 +285,10 @@ class FFTConvPDFV1(BaseFunctor):
         #     y_kernel_rect,
         #     mode='same'
         # )[None, ..., None]
-        train_points = tf.expand_dims(x_func, axis=0)
-        query_points = tf.expand_dims(x.value(), axis=0)
+        train_points = znp.expand_dims(x_func, axis=0)
+        query_points = znp.expand_dims(x.value(), axis=0)
         if self.conv_interpolation == 'spline':
-            conv_points = tf.reshape(conv, (1, -1, 1))
+            conv_points = znp.reshape(conv, (1, -1, 1))
             prob = tfa.image.interpolate_spline(train_points=train_points,
                                                 train_values=conv_points,
                                                 query_points=query_points,
